# Generated by Django 4.0.7 on 2023-10-13 12:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0011_stats_curmonth_leadergraph"),
    ]

    operations = [
        migrations.AddField(
            model_name="stats",
            name="user_mining_ledger_90day",
            field=models.JSONField(default=None, null=True),
        ),
    ]
