# Generated by Django 4.0.7 on 2022-11-22 12:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0005_oreprices_raw_price_oreprices_refined_price_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="charactertaxcredits",
            name="credit_type",
            field=models.CharField(default="", max_length=32),
        ),
    ]
