# Generated by Django 4.2.29 on 2026-03-21 21:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eve_sde", "0018_blueprintactivity_blueprintactivityproduct_and_more"),
        ("miningtaxes", "0013_refactor_tax_rates"),
    ]

    operations = [
        migrations.AlterField(
            model_name="adminminingobslog",
            name="eve_solar_system",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eve_sde.solarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="adminminingobslog",
            name="eve_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eve_sde.itemtype",
            ),
        ),
        migrations.AlterField(
            model_name="characterminingledgerentry",
            name="eve_solar_system",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eve_sde.solarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="characterminingledgerentry",
            name="eve_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eve_sde.itemtype",
            ),
        ),
        migrations.AlterField(
            model_name="oreprices",
            name="eve_type",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to="eve_sde.itemtype"
            ),
        ),
    ]
