# Generated by Django 4.0.7 on 2023-05-28 01:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0007_character_life_credits_character_life_taxes"),
    ]

    operations = [
        migrations.AddField(
            model_name="character",
            name="monthly_credits_json",
            field=models.JSONField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="character",
            name="monthly_mining_json",
            field=models.JSONField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="character",
            name="monthly_taxes_json",
            field=models.JSONField(default=None, null=True),
        ),
    ]
