default_app_config = "miningtaxes.apps.MiningTaxesConfig"

__version__ = "2.0.1"
__title__ = "Mining Taxes"

__esi_compatibility_date__ = "2025-11-06"  # ESI compatibility date, see https://esi.evetech.net/meta/compatibility-dates

__app_name__ = "miningtaxes"
__app_name_verbose__ = "Alliance Auth Mining Taxes"
__app_name_useragent__ = "AA-Mining-Taxes"

__git_repo_url__ = "https://gitlab.com/arctiru/aa-miningtaxes"
